"use strict";
/* Copyright (c) 2023 Environmental Systems Research Institute, Inc.
 * Apache-2.0 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateOAuthApp = void 0;
const helpers_js_1 = require("./shared/helpers.js");
const arcgis_rest_portal_1 = require("@esri/arcgis-rest-portal");
const arcgis_rest_request_1 = require("@esri/arcgis-rest-request");
const getRegisteredAppInfo_js_1 = require("./shared/getRegisteredAppInfo.js");
/**
 * Used to update an OAuth2.0 app.
 *
 * Notes about `redirect_uris` options:
 * 1. Provided option will override corresponding old option.
 * 2. Unprovided option will not trigger corresponding option updates.
 *
 * ```js
 * import { updateOAuthApp, IOAuthApp } from '@esri/arcgis-rest-developer-credentials';
 * import { ArcGISIdentityManager } from "@esri/arcgis-rest-request";
 *
 * const authSession: ArcGISIdentityManager = await ArcGISIdentityManager.signIn({
 *   username: "xyz_usrName",
 *   password: "xyz_pw"
 * });
 *
 * updateOAuthApp({
 *   itemId: "xyz_itemId",
 *   authentication: authSession
 * }).then((updatedOAuthApp: IOAuthApp) => {
 *   // => This OAuth app will be not be updated because its redirect_uris is not provided.
 *   // => {redirect_uris: ["http://localhost:3000/"], item: {tags: ["xyz_tag1", "xyz_tag2"], ...}, ...}
 * }).catch(e => {
 *   // => an exception object
 * });
 * ```
 *
 * @param requestOptions - Options for {@linkcode updateOAuthApp | updateOAuthApp()}, including `itemId` of which OAuth app to be operated on, optional new `redirect_uris` and an {@linkcode ArcGISIdentityManager} authentication session.
 * @returns A Promise that will resolve to an {@linkcode IOAuthApp} object representing updated OAuth app.
 */
async function updateOAuthApp(requestOptions) {
    requestOptions.httpMethod = "POST";
    // get app
    const baseRequestOptions = (0, helpers_js_1.extractBaseRequestOptions)(requestOptions); // get base requestOptions snapshot
    const getAppOption = Object.assign(Object.assign({}, baseRequestOptions), { authentication: requestOptions.authentication, itemId: requestOptions.itemId });
    const appResponse = await (0, getRegisteredAppInfo_js_1.getRegisteredAppInfo)(getAppOption);
    if (appResponse.appType === "apikey") {
        throw new Error("Item is not an OAuth 2.0 app.");
    }
    const clientId = appResponse.client_id;
    const options = (0, arcgis_rest_request_1.appendCustomParams)(Object.assign(Object.assign({}, appResponse), requestOptions), [
        "redirect_uris"
    ]);
    options.params.f = "json";
    options.params.appType = "multiple";
    // encode special params value (e.g. array type...) in advance in order to make encodeQueryString() works correctly
    (0, helpers_js_1.stringifyArrays)(options);
    const url = (0, arcgis_rest_portal_1.getPortalUrl)(options) + `/oauth2/apps/${clientId}/update`;
    // Raw response from `/oauth2/apps/${clientId}/update`.
    const updateResponse = await (0, arcgis_rest_request_1.request)(url, Object.assign(Object.assign({}, options), { authentication: requestOptions.authentication }));
    const app = (0, helpers_js_1.registeredAppResponseToApp)(updateResponse);
    const itemInfo = await (0, arcgis_rest_portal_1.getItem)(requestOptions.itemId, Object.assign(Object.assign({}, baseRequestOptions), { authentication: requestOptions.authentication, params: { f: "json" } }));
    return Object.assign(Object.assign({}, (0, helpers_js_1.appToOAuthAppProperties)(app)), { item: itemInfo });
}
exports.updateOAuthApp = updateOAuthApp;
//# sourceMappingURL=updateOAuthApp.js.map